package com.baidu.rtc.videoroom;

import com.baidu.rtc.BaiduRtcRoom;
import com.baidu.rtc.RTCVideoView;
import com.baidu.rtc.RtcParameterSettings;

import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.hardware.camera2.CameraManager;
import android.os.Bundle;
import android.view.View;
import android.widget.GridLayout;

public class DuoCapturesCallActivity extends RtcCommonActivity {
    private static final String TAG = "DuoCapturesCallActivity";

    protected BaiduRtcRoom mVideoRoom2;
    protected BaiduRtcRoom mVideoRoom3;
    protected BaiduRtcRoom mVideoRoom4;
    private int mCameraCount = 0;

    /**
     * 判断是否平板设备
     *
     * @param context
     *
     * @return true:平板,false:手机
     */
    private boolean isTabletDevice(Context context) {
        return (context.getResources().getConfiguration().screenLayout
                        & Configuration.SCREENLAYOUT_SIZE_MASK) >=
                Configuration.SCREENLAYOUT_SIZE_LARGE;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_duocapturescall);
        rootView = (GridLayout) findViewById(R.id.activity_videocall);

        mVideoRoom.setLocalDisplay((RTCVideoView) findViewById(R.id.local_rtc_video_view));
        //        mVideoRoom.setVerbose(true);

        RtcParameterSettings cfg = RtcParameterSettings.getDefaultSettings();
        cfg.HasAudio = false;
        cfg.VideoResolution = mVideoResolution;
        cfg.VideoCodec = "h264";
        cfg.VideoFps = 20;
        cfg.HasData = true;
        mVideoRoom.setParamSettings(cfg,
                RtcParameterSettings.RtcParamSettingType.RTC_PARAM_SETTINGS_ALL);

        if (mVideoResolution.contains("1000kbps")) {
            cfg.VideoMaxkbps = 1000;
        } else if (mVideoResolution.contains("1500kbps")) {
            cfg.VideoMaxkbps = 1500;
        } else if (mVideoResolution.contains("2000kbps")) {
            cfg.VideoMaxkbps = 2000;
        } else if (mVideoResolution.contains("3000kbps")) {
            cfg.VideoMaxkbps = 3000;
        } else {
            cfg.VideoMaxkbps = 1000;
        }

        int idx = 0;
        if (isTabletDevice(getBaseContext())) {
            idx = 2;
        }

        mVideoRoom.setCameraID(0 + idx);
        mVideoRoom.setParamSettings(cfg,
                RtcParameterSettings.RtcParamSettingType.RTC_VIDEO_PARAM_SETTINGS_BITRATE);

        mVideoRoom.loginRtcRoomWithRoomName(mRoomName, Integer.parseInt(mUserId), mUserName, true,
                false);

        CameraManager cm = (CameraManager) getSystemService(Context.CAMERA_SERVICE);

        try {
            mCameraCount = cm.getCameraIdList().length;
        } catch (Exception e) {

        }

        if (mCameraCount > 1) {
            mCameraCount = 2;
            if (!isTabletDevice(getBaseContext())) { // phone only can use one Camera at a time.
                mCameraCount = 1;
            }
        }

        if (mCameraCount > 1) {
            mVideoRoom2 = BaiduRtcRoom.initWithAppID(this, mAppId, mTokenStr);
            mVideoRoom2.setBaiduRtcRoomDelegate(this);
            mVideoRoom2.setMediaServerURL(mMediaServer);
            mVideoRoom2.setLocalDisplay((RTCVideoView) findViewById(R.id.local2_rtc_video_view));
            mVideoRoom2.setCameraID(1 + idx);
            mVideoRoom2.setParamSettings(cfg,
                    RtcParameterSettings.RtcParamSettingType.RTC_PARAM_SETTINGS_ALL);
            mVideoRoom2.setParamSettings(cfg,
                    RtcParameterSettings.RtcParamSettingType.RTC_VIDEO_PARAM_SETTINGS_BITRATE);
            mVideoRoom2.loginRtcRoomWithRoomName(mRoomName + ".2", Integer.parseInt(mUserId) + 1,
                    mUserName, true, false);
        }

        if (mCameraCount > 2) {
            mVideoRoom3 = BaiduRtcRoom.initWithAppID(this, mAppId, mTokenStr);
            mVideoRoom3.setBaiduRtcRoomDelegate(this);
            mVideoRoom3.setMediaServerURL(mMediaServer);
            mVideoRoom3.setLocalDisplay((RTCVideoView) findViewById(R.id.local3_rtc_video_view));
            mVideoRoom3.setCameraID(2 - idx);
            mVideoRoom3.setParamSettings(cfg,
                    RtcParameterSettings.RtcParamSettingType.RTC_PARAM_SETTINGS_ALL);
            mVideoRoom3.setParamSettings(cfg,
                    RtcParameterSettings.RtcParamSettingType.RTC_VIDEO_PARAM_SETTINGS_BITRATE);
            mVideoRoom3.loginRtcRoomWithRoomName(mRoomName + ".3", Integer.parseInt(mUserId) + 2,
                    mUserName, true, false);
        }

        if (mCameraCount > 3) {
            mVideoRoom4 = BaiduRtcRoom.initWithAppID(this, mAppId, mTokenStr);
            mVideoRoom4.setBaiduRtcRoomDelegate(this);
            mVideoRoom4.setMediaServerURL(mMediaServer);
            mVideoRoom4.setLocalDisplay((RTCVideoView) findViewById(R.id.local4_rtc_video_view));
            mVideoRoom4.setCameraID(3 - idx);
            mVideoRoom4.setParamSettings(cfg,
                    RtcParameterSettings.RtcParamSettingType.RTC_PARAM_SETTINGS_ALL);
            mVideoRoom4.setParamSettings(cfg,
                    RtcParameterSettings.RtcParamSettingType.RTC_VIDEO_PARAM_SETTINGS_BITRATE);
            mVideoRoom4.loginRtcRoomWithRoomName(mRoomName + ".4", Integer.parseInt(mUserId) + 3,
                    mUserName, true, false);
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        mVideoRoom.muteMicphone(true);
        if (mCameraCount > 1) {
            mVideoRoom2.startPreview();
        }
        if (mCameraCount > 2) {
            mVideoRoom3.startPreview();
        }
        if (mCameraCount > 3) {
            mVideoRoom4.startPreview();
        }
    }

    @Override
    protected void onDestroy() {
        if (mCameraCount > 1) {
            mVideoRoom2.destroy();
        }
        if (mCameraCount > 2) {
            mVideoRoom3.destroy();
        }
        if (mCameraCount > 3) {
            mVideoRoom4.destroy();
        }
        super.onDestroy();
    }

    public void onEndCallClicked(View view) {
        if (mCameraCount > 1) {
            mVideoRoom2.logoutRtcRoom();
            mVideoRoom2.destroy();
        }
        if (mCameraCount > 2) {
            mVideoRoom3.logoutRtcRoom();
            mVideoRoom3.destroy();
        }
        if (mCameraCount > 3) {
            mVideoRoom4.logoutRtcRoom();
            mVideoRoom4.destroy();
        }
        super.onEndCallClicked(view);
    }

    public void onBacktoHome(View view) {
        Intent intent = new Intent();
        intent.setAction(Intent.ACTION_MAIN);
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); // 如果是服务里调用，必须加入new task标识
        intent.addCategory(Intent.CATEGORY_HOME);
        startActivity(intent);
    }
}
