package com.baidu.rtc.videoroom;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.telecom.InCallService;
import android.view.View;
import android.widget.GridLayout;

import com.baidu.rtc.BaiduRtcRoom;
import com.baidu.rtc.RTCVideoFrame;
import com.baidu.rtc.RTCVideoView;
import android.util.Log;
import android.widget.ImageView;
import android.widget.Toast;

import org.webrtc.VideoFrame;

import java.nio.ByteBuffer;


public class VideoCallActivity extends RtcCommonActivity implements VideoCallProtocol.CallStateObserver{
    private static final String TAG = "VideoCallActivity";

    myOuterRTCVideoCapture selfRTCVideoCapture = null;
    AlertDialog callDialog;
    AlertDialog receiveDialog;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        VideoCallProtocol.CallConfig config = new VideoCallProtocol.CallConfig();
        callProtocol = new VideoCallProtocol(this,config);

        setContentView(R.layout.activity_videocall);
        rootView = (GridLayout) findViewById(R.id.activity_videocall);
        callBtnImage = (ImageView) findViewById(R.id.call_imageview);
        if (callMode) {
            callBtnImage.setImageResource(R.drawable.btn_start_call);
            callBtnImage.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    if (callProtocol.getCurrentState() == VideoCallProtocol.CallState.kStable) {
                        callProtocol.startCall();
                        callBtnImage.setImageResource(R.drawable.btn_end_call);
                        //
                        final AlertDialog.Builder dialog = new AlertDialog.Builder(VideoCallActivity.this);
                        dialog.setTitle(R.string.start_call);
                        dialog.setMessage(R.string.in_calling_state);
                        dialog.setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialogInterface, int i) {
                                callProtocol.cancelCall();
                            }
                        });
                        callDialog = dialog.show();
                    }else if (callProtocol.getCurrentState() == VideoCallProtocol.CallState.kCalling) {
                        callProtocol.finishCall();
                        callBtnImage.setImageResource(R.drawable.btn_start_call);
                    }else{
                        callProtocol.cancelCall();
                        callBtnImage.setImageResource(R.drawable.btn_start_call);
                    }
                }
            });
        }

        mVideoRoom.setLocalDisplay((RTCVideoView) findViewById(R.id.local_rtc_video_view));
        mVideoRoom.setRemoteDisplay((RTCVideoView) findViewById(R.id.remote_rtc_video_view));
//
//        RTCVideoView l = new RTCVideoView(this);
//        l.setExtVideoSink(new RTCVideoView.ExtVideoSink() {
//            @Override
//            public void onFrame(VideoFrame videoFrame) {
//
//                ByteBuffer y = videoFrame.getBuffer().toI420().getDataY();
//
//                Log.e("RTCVideoView", "y data is: " + y.get(100) + y.get(101));
//            }
//        });
//        mVideoRoom.setLocalDisplay(l);
//
//
//        RTCVideoView r = new RTCVideoView(this);
//        r.setExtVideoSink(new RTCVideoView.ExtVideoSink() {
//            @Override
//            public void onFrame(VideoFrame videoFrame) {
//
//                ByteBuffer y = videoFrame.getBuffer().toI420().getDataY();
//
//                Log.e("RTCVideoView", "y data is: " + y.get(100) + y.get(101));
//            }
//        });
//        mVideoRoom.setRemoteDisplay(r);


//        selfRTCVideoCapture = new myOuterRTCVideoCapture();
//
//        mVideoRoom.setVideoCapture(selfRTCVideoCapture);
//
//        mVideoRoom.setRemoteDisplay(new RTCVideoView(this) {
//            @Override
//            public void onRTCVideoFrame(RTCVideoFrame frame) {
//
//                Log.e(TAG,"got video data TS: "+frame.getTimestampNs());
//            }
//            @Override
//            public boolean onlyforVideoCallbackdata() {return true;}
//        });
//        mVideoRoom.setAuidoSamplesReadyCallback((samples)->{
//            Log.e(TAG,"got audio data SR: "+samples.getSampleRate());
//        });

        DoLogin();
    }

    @Override
    public void onStateChange(VideoCallProtocol.CallState state, VideoCallProtocol.CallRole role,
                              VideoCallProtocol.CallCommand reasonCommand, VideoCallProtocol.CallRole commandSource) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                switch (state) {
                    case kStable:
                        if (role == commandSource && reasonCommand == VideoCallProtocol.CallCommand.kCancel) {
                            if (role == VideoCallProtocol.CallRole.kSender)
                                callDialog.dismiss();
                            else
                                receiveDialog.dismiss();
                            Toast.makeText(getApplicationContext(), R.string.canceled_call,Toast.LENGTH_LONG).show();
                        }else if (role != commandSource && reasonCommand == VideoCallProtocol.CallCommand.kCancel) {
                            if (role == VideoCallProtocol.CallRole.kSender) {
                                callDialog.dismiss();
                                Toast.makeText(getApplicationContext(),
                                        R.string.refused_call, Toast.LENGTH_LONG).show();
                            }else {
                                receiveDialog.dismiss();
                                Toast.makeText(getApplicationContext(),
                                        R.string.other_canceled_call, Toast.LENGTH_LONG).show();
                            }
                        }
                        else if (role == VideoCallProtocol.CallRole.kSender &&
                                reasonCommand == VideoCallProtocol.CallCommand.kRequestTimeout) {
                            callDialog.dismiss();
                            Toast.makeText(getApplicationContext(), R.string.not_response_end,Toast.LENGTH_LONG).show();
                        }else if (role == VideoCallProtocol.CallRole.kSender &&
                                reasonCommand == VideoCallProtocol.CallCommand.kBusyHere) {
                            callDialog.dismiss();
                            Toast.makeText(getApplicationContext(),
                                    R.string.in_call_please_wait,Toast.LENGTH_LONG).show();
                        }else if (role == commandSource && reasonCommand == VideoCallProtocol.CallCommand.kBye) {
                            Toast.makeText(getApplicationContext(), R.string.end_call_over,Toast.LENGTH_LONG).show();
                            mVideoRoom.stopPublish();
                            mVideoRoom.stopSubscribeStreaming(partnerId);
                        }else if (role != commandSource && reasonCommand == VideoCallProtocol.CallCommand.kBye) {
                            Toast.makeText(getApplicationContext(),
                                    R.string.other_end_call_over,Toast.LENGTH_LONG).show();
                            mVideoRoom.stopPublish();
                            mVideoRoom.stopSubscribeStreaming(partnerId);
                        }
                        else if (role == VideoCallProtocol.CallRole.kReceiver &&
                                reasonCommand == VideoCallProtocol.CallCommand.kRequestTimeout) {
                            receiveDialog.dismiss();
                            Toast.makeText(getApplicationContext(),
                                    R.string.timeout_end, Toast.LENGTH_LONG).show();
                        }
                        callBtnImage.setImageResource(R.drawable.btn_start_call);
                        break;
                    case kInviting:
                        break;
                    case kRinging:
                        if (role == VideoCallProtocol.CallRole.kReceiver) {
                            onReceiveInvite();
                        }else{
                            callDialog.setMessage(getString(R.string.wait_accept));
                        }
                        break;
                    case kCalling:
                        //
                        if (role == VideoCallProtocol.CallRole.kSender) {
                            callDialog.dismiss();
                            Toast.makeText(getApplicationContext(),
                                    R.string.accept_establish,Toast.LENGTH_SHORT).show();
                        }else{
                            Toast.makeText(getApplicationContext(),
                                    R.string.call_establish_ing,Toast.LENGTH_SHORT).show();
                        }
                        //
                        callBtnImage.setImageResource(R.drawable.btn_end_call);
                        mVideoRoom.startPublish();
                        mVideoRoom.subscribeStreaming(0,partnerId);
                        break;
                    case kReceiveInviting:
                        break;
                }
            }
        });
    }

    @Override
    public void sendMessage(String msg) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                mVideoRoom.sendMessageToUser(msg,partnerId);
            }
        });
    }

    private void onReceiveInvite() {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                final AlertDialog.Builder dialog = new AlertDialog.Builder(VideoCallActivity.this);
                dialog.setTitle(R.string.receive_call);
                dialog.setMessage(R.string.make_sure_accept_call);
                dialog.setPositiveButton(R.string.answer, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        callProtocol.receiveCall();
                    }
                });
                dialog.setNegativeButton(R.string.refuse_call, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        callProtocol.cancelCall();
                    }
                });
                receiveDialog = dialog.show();
            }
        });
    }
}
